/*
 * This file is part of BukkitHTTPD.
 * Copyright (C) 2011 "codename_B" Ben Smith <BenBenBen001@googlemail.com>
 * Copyright (C) 2011 Thomas "V10lator" Rohloff <v10lator@myway.de>
 * 
 *
 * BukkitHTTPD is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * BukkitHTTPD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BukkitHTTPD.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.V10lator.BukkitHTTPD;

import java.io.File;
import java.nio.CharBuffer;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.Charset;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import org.bukkit.ChatColor;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

/**
 * 
 * @author V10lator
 * 
 */
public class BukkitHTTPDCommandParser implements CommandExecutor
{
  private final BukkitHTTPD plugin;
  
  public BukkitHTTPDCommandParser(BukkitHTTPD plugin)
  {
	this.plugin = plugin;
  }
  
  public boolean onCommand(CommandSender sender, Command arg1, String arg2,
			String[] args)
  {
	boolean pw = (args.length > 1 && args[0].equalsIgnoreCase("password"));
	if(!sender.hasPermission("bukkithttpd.admin") || pw)
	{
	  if(!sender.hasPermission("bukkithttpd.user") || !(sender instanceof Player))
	    return true;
	  if(!pw)
	    sendHelp(sender);
	  else
	  {
		String passwd;
		if(args.length > 2)
		{
		  StringBuilder sb = new StringBuilder(args[1]);
		  for(int i = 2; i < args.length; i++)
		  {
			sb.append(args[i]);
			sb.append(" ");
		  }
		  passwd = sb.toString().trim();
		}
		else
		  passwd = args[1].trim();
		try
		{
		  MessageDigest md = MessageDigest.getInstance("SHA");
		  md.update(Charset.forName("UTF-8").newEncoder().encode(CharBuffer.wrap(passwd)));
		  byte[] sha = md.digest();
		  StringBuffer hex = new StringBuffer();
		  for(int i=0; i < sha.length; i++)
		    hex.append(Integer.toHexString(0xFF & sha[i]));
		  
		  while(plugin.ram.lock.compareAndSet(false, true))
			continue;
		  String sn = ((Player)sender).getName();
		  plugin.ram.passwords.put(sn, hex.toString());
		  if(!plugin.admins.containsKey(sn))
		    plugin.admins.put(sn, false);
		  plugin.ram.sessions.remove(sn);
		  plugin.ram.lock.set(false);
		  plugin.saveConfig();
		  sender.sendMessage(ChatColor.GREEN+"Password set.");
		}
		//TODO
		catch(CharacterCodingException e)
		{
		  sender.sendMessage(ChatColor.RED+"Internal error!");
		  e.printStackTrace();
		}
		catch(NoSuchAlgorithmException e)
		{
		  sender.sendMessage(ChatColor.RED+"Internal error!");
		  e.printStackTrace();
		}
	  }
	  return true;
	}
	
	while(plugin.ram.lock.compareAndSet(false, true))
	  continue;
	
	boolean save = false;
	boolean reload = false;
	
	if(args.length < 1 || args[0].equalsIgnoreCase("help"))
	  sendHelp(sender);
	else if(args[0].equalsIgnoreCase("name"))
	{
	  if(args.length < 2)
	  {
		sender.sendMessage(ChatColor.GREEN+"Current server name:");
		sender.sendMessage(ChatColor.YELLOW+plugin.ram.serverName);
	  }
	  else
	  {
		StringBuilder sb = new StringBuilder(args[1]);
		for(int i = 2; i < args.length; i++)
		{
		  sb.append(" ");
		  sb.append(args[i]);
		}
		plugin.ram.serverName = sb.toString();
		save = true;
		sender.sendMessage(ChatColor.YELLOW+"New server name:");
		sender.sendMessage(ChatColor.GREEN+plugin.ram.serverName);
	  }
	}
	else if(args[0].equalsIgnoreCase("port"))
	{
	  if(args.length < 2)
	  {
		sender.sendMessage(ChatColor.GREEN+"Current server port:");
		sender.sendMessage(ChatColor.YELLOW+String.valueOf(plugin.httpdPort)+" ("+ChatColor.GOLD+plugin.ram.hidePort+ChatColor.YELLOW+")");
	  }
	  else
	  {
		try
		{
		  if(args[1].equalsIgnoreCase("hide"))
		  {
			if(plugin.ram.hidePort)
			{
			  plugin.ram.hidePort = false;
			  sender.sendMessage(ChatColor.YELLOW+"Hiding port.");
			}
			else
			{
			  plugin.ram.hidePort = true;
			  sender.sendMessage(ChatColor.GREEN+"Showing port.");
			}
		  }
		  else
		  {
		    plugin.httpdPort = Integer.parseInt(args[1]);
		    sender.sendMessage(ChatColor.YELLOW+"New server port:");
		    sender.sendMessage(ChatColor.GREEN+args[1]);
		  }
		  save = true;
		}
		catch(NumberFormatException e)
		{
		  sender.sendMessage(ChatColor.RED+"Invalid port: \""+ChatColor.GOLD+args[1]+ChatColor.RED+"\"");
		}
	  }
	}
	else if(args[0].equalsIgnoreCase("color"))
	{
	  if(args.length < 2)
	  {
		sender.sendMessage(ChatColor.GREEN+"Current color:");
		sender.sendMessage(ChatColor.YELLOW+plugin.ram.color);
	  }
	  else
	  {
		try
		{
		  Integer.parseInt(args[1]);
		  if(args[1].length() != 6)
			throw new NumberFormatException();
		  plugin.ram.color = args[1];
		  save = true;
		  sender.sendMessage(ChatColor.YELLOW+"New color:");
		  sender.sendMessage(ChatColor.GREEN+plugin.ram.color);
		}
		catch(NumberFormatException e)
		{
		  sender.sendMessage(ChatColor.RED+"Invalid color: \""+ChatColor.GOLD+args[1]+ChatColor.RED+"\"");
		}
	  }
	}
	else if(args[0].equalsIgnoreCase("refresh"))
	{
	  try
	  {
		int r = Integer.parseInt(args[1]);
		if(r < 1)
		  sender.sendMessage(ChatColor.RED+"Invalid refresh rate: \""+ChatColor.GOLD+args[1]+ChatColor.RED+"\"");
		else
		{
		  plugin.itimer = r;
		  sender.sendMessage(ChatColor.GREEN+"Refresh rate set!");
		  save = true;
		}
	  }
	  catch(NumberFormatException e)
	  {
		sender.sendMessage(ChatColor.RED+"Invalid refresh rate: \""+ChatColor.GOLD+args[1]+ChatColor.RED+"\"");
	  }
	}
	else if(args[0].equalsIgnoreCase("dir"))
	{
	  if(args.length < 2)
	  {
		sender.sendMessage(ChatColor.GREEN+"Current directory:");
		sender.sendMessage(ChatColor.YELLOW+plugin.wwwDir.toString());
	  }
	  else
	  {
		StringBuilder sb = new StringBuilder(args[1]);
		for(int i = 2; i < args.length; i++)
		{
		  sb.append(" ");
		  sb.append(args[i]);
		}
		plugin.wwwDir = new File(sb.toString());
		save = true;
		sender.sendMessage(ChatColor.YELLOW+"New directory:");
		sender.sendMessage(ChatColor.GREEN+plugin.wwwDir.toString());
	  }
	}
	else if(args[0].equalsIgnoreCase("bans"))
	{
	  if(plugin.ram.minecraftBans)
	  {
		plugin.ram.minecraftBans = false;
		sender.sendMessage(ChatColor.YELLOW+"Ignoring minecraft bans.");
	  }
	  else
	  {
		plugin.ram.minecraftBans = true;
		sender.sendMessage(ChatColor.GREEN+"Using minecraft bans.");
	  }
	}
	else if(args[0].equalsIgnoreCase("admin") && args.length > 1)
	{
	  if(!plugin.ram.passwords.containsKey(args[1]))
		sender.sendMessage(ChatColor.RED+args[1]+" not found!");
	  else
	  {
		if(plugin.admins.get(args[1]))
		{
		  plugin.admins.put(args[1], false);
		  sender.sendMessage(ChatColor.YELLOW+args[1]+" is no longer web admin.");
		}
		else
		{
		  plugin.admins.put(args[1], true);
		  sender.sendMessage(ChatColor.YELLOW+args[1]+" is web admin now.");
		}
		save = true;
	  }
	}
	else if(args[0].equalsIgnoreCase("reload"))
	  reload = true;
	//TODO: Enable after bugfix.
	/*else if(args[0].equalsIgnoreCase("rmpwd") && args.length > 1)
	{
	  if(plugin.ram.passwords.containsKey(args[1]))
	  {
		plugin.ram.passwords.remove(args[1]);
		plugin.ram.loggedIn.remove(args[1]);
		save = true;
	  }
	  else
		sender.sendMessage(ChatColor.RED+args[1]+" not found!");
	}*/
	else
	  sendHelp(sender);
	
	plugin.ram.lock.set(false);
	if(save)
	{
	  plugin.saveConfig();
	  plugin.reloadConfig();
	  sender.sendMessage(ChatColor.GREEN+"Config saved!");
	}
	else if(reload)
	{
	  plugin.reloadConfig();
	  plugin.reloadConfig();
	  sender.sendMessage(ChatColor.GREEN+"Config reloaded!");
	}
	return true;
  }
  
  private void sendHelp(CommandSender sender)
  {
	if(sender.hasPermission("bukkithttpd.admin"))
	{
	  sender.sendMessage("/httpd name");
	  sender.sendMessage("    Show/set the server name.");
	  sender.sendMessage("/httpd port");
	  sender.sendMessage("    Show/set the server port.");
	  sender.sendMessage("/httpd port hide");
	  sender.sendMessage("    Toggle the port hiding switch.");
	  sender.sendMessage("/httpd color");
	  sender.sendMessage("    Show/set the banner color.");
	  sender.sendMessage("/httpd refrest");
	  sender.sendMessage("    set the banner refresh rate.");
	  sender.sendMessage("/httpd dir");
	  sender.sendMessage("    Show/set the www folder.");
	  sender.sendMessage("/httpd bans");
	  sender.sendMessage("    Toggle the minecraft bans feature.");
	  sender.sendMessage("/httpd reload");
	  sender.sendMessage("    Reload the configuration from disk.");
	  //TODO: Enable after bugfix.
	  //sender.sendMessage("/httpd rmpwd <User>");
	  //sender.sendMessage("    Delete the BukkitHTTPD password of user <User>.");
	}
	if(sender.hasPermission("bukkithttpd.user") || !(sender instanceof Player))
	{
	  sender.sendMessage("/httpd password <password>");
	  sender.sendMessage("    Set your BukkitHTTPD password.");
	}
	sender.sendMessage("/httpd help");
	sender.sendMessage("    Show this help.");
  }
}
