/*
 * This file is part of BukkitHTTPD.
 * Copyright (C) 2011 "codename_B" Ben Smith <BenBenBen001@googlemail.com>
 * Copyright (C) 2011 Thomas "V10lator" Rohloff <v10lator@myway.de>
 * 
 *
 * BukkitHTTPD is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * BukkitHTTPD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BukkitHTTPD.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.V10lator.BukkitHTTPD;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.nio.CharBuffer;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.Charset;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.UUID;

import org.bukkit.ChatColor;

/**
 * 
 * @author V10lator
 *
 */
public class V10parser
{
  private final RAM ram;
  
  public V10parser(RAM ram)
  {
	this.ram = ram;
  }
  
  public String parse(File f, Properties parms, String ip)
  {
	  String content;
	  String prefix = "<h2>V10parse error:</h2>\n";
	  try
	  {
		BufferedReader reader = new BufferedReader(new FileReader(f));
		char[] buffer = new char[(int)f.length()];
		if(reader.read(buffer) < 0)
		  return prefix+"Unable to read file!";
		
		content = String.valueOf(buffer);
	  }
	  catch(IOException e)
	  {
		return prefix+"Unable to read file!";
	  }
	  
	  //dos2unix:
	  content.replace("\r\n", "\n");
	  
	  //Make copy:
	  String worker = content;
	  
	  //Newline to space:
	  worker.replace("\n", " ");
	  
	  //Tabs to space:
	  worker.replace("\t", " ");
	  
	  // find v10 marks:
	  while(worker.contains("<?v10 "))
	  {
		int start = worker.indexOf("<?v10 "); 
		int end = worker.indexOf(" ?>");
		if(end < 0)
		  return prefix+"Unclosed V10code block!";
		while(end < start)
		{
		  end = worker.indexOf(" ?>", end + 1);
		  if(end < 0)
			return prefix+"Unclosed V10code block";
		}
		
		String codeblock = worker.substring(start + 6, end).trim();
		// TODO: Extend parser...
		while(!ram.lock.compareAndSet(false, true))
		  continue;
		
		String replace = null;
		StringBuilder sb = new StringBuilder(" = new Array(");
		if(codeblock.equalsIgnoreCase("players"))
		{
		  if(!ram.players.isEmpty())
		  {
		    for(String p: ram.players)
		    {
			  sb.append("\"");
			  sb.append(p);
			  sb.append("\", ");
		    }
			int l = sb.length();
			if(l > 4)
			  sb.delete(l - 2, l);
		  }
		  else
			sb.append("\"None\"");
		  sb.insert(0, "players");
		}
		else if(codeblock.equalsIgnoreCase("maxPlayers"))
		{
		  sb.append("\"");
		  sb.append(ram.maxPlayers);
		  sb.append("\"");
		  sb.insert(0, "maxPlayers");
		}
		else if(codeblock.equalsIgnoreCase("worlds"))
		{
		  if(!ram.worlds.isEmpty())
		  {
		    for(String w: ram.worlds)
		    {
			  sb.append("\"");
			  sb.append(w);
			  sb.append("\", ");
		    }
			int l = sb.length();
			if(l > 4)
			  sb.delete(l - 2, l);
		  }
		  else
			sb.append("\"None\"");
		  sb.insert(0, "worlds");
		}
		else if(codeblock.equalsIgnoreCase("plugins"))
		{
		  if(!ram.plugins.isEmpty())
		  {
		    for(String p: ram.plugins)
		    {
			  sb.append("\"");
			  sb.append(p);
			  sb.append("\", ");
		    }
		    int l = sb.length();
		    if(l > 4)
			  sb.delete(l - 2, l);
		  }
		  else
			sb.append("\"None\"");
		  sb.insert(0, "plugins");
		}
		else if(codeblock.equalsIgnoreCase("bannedPlayers"))
		{
		  if(!ram.bannedPlayers.isEmpty())
		  {
		    for(String bp: ram.bannedPlayers)
		    {
			  sb.append("\"");
			  sb.append(bp);
			  sb.append("\", ");
		    }
			int l = sb.length();
			if(l > 4)
			  sb.delete(l - 2, l);
		  }
		  else
			sb.append("\"None\"");
		  sb.insert(0, "bannedPlayers");
		}
		else if(codeblock.equalsIgnoreCase("chat"))
		{
		  if(!ram.chatPtH.isEmpty())
		  {
			long ts;
		    for(Entry<UUID, String> e: ((HashMap<UUID, String>)ram.chatPtH.clone()).entrySet())
		    {
		      UUID uuid = e.getKey();
		      ts = ram.chatMappings.get(uuid);
		      if((System.currentTimeMillis() / 1000) - ts > 60)
		      {
		    	ram.chatMappings.remove(uuid);
		    	ram.chatPtH.remove(uuid);
		    	//continue;
		      }
			  sb.append("\"|");
			  sb.append(ts);
			  sb.append("| ");
			  sb.append(e.getValue());
			  sb.append("\", ");
		    }
			int l = sb.length();
			if(l > 4)
			  sb.delete(l - 2, l);
		  }
		  else
			sb.append("\"None\"");
		  sb.insert(0, "chat");
		}
		else if(codeblock.equalsIgnoreCase("gameMode"))
		{
		  sb.append("\"");
		  sb.append(ram.gameMode);
		  sb.append("\"");
		  sb.insert(0, "gameMode");
		}
		else if(codeblock.equalsIgnoreCase("ip"))
		{
		  sb.append("\"");
		  sb.append(ip);
		  sb.append("\", ");
		  sb.insert(0, "ip");
		}
		else if(codeblock.equalsIgnoreCase("ipBans"))
		{
		  if(!ram.ipBans.isEmpty())
		  {
		    for(String ib: ram.ipBans)
		    {
			  sb.append("\"");
			  sb.append(ib);
			  sb.append("\", ");
		    }
			int l = sb.length();
			if(l > 4)
			  sb.delete(l - 2, l);
		  }
		  else
			sb.append("\"None\"");
		  sb.insert(0, "ipBans");
		}
		else if(codeblock.equalsIgnoreCase("serverName"))
		{
		  if(ram.serverName != null)
		  {
		    sb.append("\"");
			sb.append(ram.serverName);
			sb.append("\"");
		  }
		  else
			sb.append("\"N/A\"");
		  sb.insert(0, "serverName");
		}
		else if(codeblock.equalsIgnoreCase("serverHost"))
		{
		  if(ram.serverHost != null)
		  {
		    sb.append("\"");
			sb.append(ram.serverHost);
			sb.append("\"");
		  }
		  else
			sb.append("\"N/A\"");
		  sb.insert(0, "serverHost");
		}
		else if(codeblock.equalsIgnoreCase("color"))
		{
		  if(ram.color != null)
		  {
		    sb.append("\"");
			sb.append(ram.color);
			sb.append("\"");
		  }
		  else
			sb.append("\"N/A\"");
		  sb.insert(0, "color");
		}
		else if(codeblock.equalsIgnoreCase("login"))
		{
		  String p = parms.getProperty("login", "§§");
		  if(p.equals("§§") || p.equals(""))
		    return exit(prefix+"4: Permissions error: Wrong username!");
		  if(ram.minecraftBans && ram.bannedPlayers.contains(p))
			return exit(prefix+"6: Permissions error: Username baned!");
		  String pw = parms.getProperty("password", null);
		  if(pw == null)
			return exit(prefix+"5: Permissions error: Wrong password!");
		  BukkitHTTPDSession session;
		  if(ram.sessions.containsKey(p))
		  {
		    session = ram.sessions.get(p);
		    try
		    {
		      if(!session.hash.equals(pw))
		        return exit(prefix+"5: Permissions error: Wrong password!");
		    }
		    catch(IllegalArgumentException e)
		    {
		      return exit(prefix+"5: Permissions error: Wrong password!");
		    }
		    if(!ip.equals(session.ip))
		      return exit(prefix+"8: Permissions error: Out of session!");
		    session.hash = UUID.randomUUID().toString();
		    session.lastUsed = System.currentTimeMillis();
		  }
		  else
		  {
			if(!ram.passwords.containsKey(p))
			  return exit(prefix+"4: Permissions error: Wrong username!");
			try
			{
			  MessageDigest md;
			  md = MessageDigest.getInstance("SHA");
			  md.update(Charset.forName("UTF-8").newEncoder().encode(CharBuffer.wrap(pw.trim())));
			  byte[] sha = md.digest();
			  StringBuffer hex = new StringBuffer();
			  for(int i=0; i < sha.length; i++)
				hex.append(Integer.toHexString(0xFF & sha[i]));
			  if(!ram.passwords.get(p).equals(hex.toString()))
				return exit(prefix+"5: Permissions error: Wrong password!");
			}
			catch(NoSuchAlgorithmException e)
			{
			  e.printStackTrace();
			  return exit(prefix+"7: Internal permissions error!");
			}
			catch(CharacterCodingException e)
			{
			  e.printStackTrace();
			  return exit(prefix+"7: Internal permissions error!");
			}
			session = new BukkitHTTPDSession(ip, UUID.randomUUID().toString(), false);
			ram.sessions.put(p, session);
		  }
		  sb.append("\"");
		  sb.append(p);
		  sb.append("\", ");
		  sb.append("\"");
		  sb.append(session.hash);
		  sb.append("\", ");
		  sb.append("\"");
		  sb.append(session.isAdmin);
		  sb.append("\"");
		  
		  sb.insert(0, "login");
		}
		else if(codeblock.equalsIgnoreCase("logout"))
		{
		  String p = parms.getProperty("login", "§§");
		  if(p.equals("§§") || p.equals(""))
		    return exit(prefix+"4: Permissions error: Wrong username!");
		  String pw = parms.getProperty("password", null);
		  if(pw == null)
			return exit(prefix+"5: Permissions error: Wrong password!");
		  if(ram.sessions.containsKey(p))
		  {
		    if(!ram.sessions.get(p).hash.equals(pw))
		      return exit(prefix+"5: Permissions error: Wrong password!");
		  }
		  else
			return exit(prefix+"5: Permissions error: Wrong password!");
		  ram.sessions.remove(p);
		  sb.insert(0, "logout");
		}
		else if(codeblock.toLowerCase().startsWith("input(") &&
				codeblock.toLowerCase().endsWith(")"))
		{
		  String input = codeblock.substring(codeblock.indexOf('(') + 1, codeblock.indexOf(')'));
		  String[] args;
		  if(input.contains(", "))
			args = input.split(", ");
		  else
			args = new String[] {input};
		  for(String arg: args)
		  {
			if(arg.equalsIgnoreCase("chat"))
			{
			  String chat = parms.getProperty("chat", "");
			  String[] chatLines;
			  if(chat.contains("\n"))
				chatLines = chat.split("\n");
			  else
				chatLines = new String[] {chat};
			  String p = parms.getProperty("login", "§§");
			  if(!p.equals("§§") && !p.equals(""))
			  {
				if(ram.minecraftBans && ram.bannedPlayers.contains(p))
				  return exit(prefix+"6: Permissions error: Username baned!");
				if(!ram.sessions.containsKey(p))
				  return exit(prefix+"8: Permissions error: Out of session!");
				ArrayList<String> tmp = new ArrayList<String>();
				for(String line: chatLines)
				{
				  if(line.length() == 0)
					continue;
				  tmp.add(ChatColor.AQUA+p+ChatColor.WHITE+"> "+line);
				}
				chatLines = (String[])tmp.toArray(new String[0]);
			  }
			  int c = 0;
			  for(String line: chatLines)
			  {
				if(line.length() == 0)
				  continue;
				ram.chatHtP.add(line);
				UUID uuid = UUID.randomUUID();
				ram.chatMappings.put(uuid, System.currentTimeMillis() / 1000);
				ram.chatPtH.put(uuid, line);
				c++;
			  }
			  sb.append("\"chat:");
			  sb.append(c);
			  sb.append("\", ");
			}
			else if(arg.equalsIgnoreCase("reboot"))
			{
			  String p = parms.getProperty("login", "§§");
			  if(p.equals("§§") || p.equals(""))
			    return exit(prefix+"4: Permissions error: Wrong username!");
			  String pw = parms.getProperty("password", null);
			  if(pw == null)
				return exit(prefix+"5: Permissions error: Wrong password!");
			  if(ram.sessions.containsKey(p))
			  {
			    if(!ram.sessions.get(p).hash.equals(pw))
			      return exit(prefix+"5: Permissions error: Wrong password!");
			  }
			  else
				return exit(prefix+"5: Permissions error: Wrong password!");
			  ram.reboot = true;
			  sb.append("\"reboot\", ");
			}
			else if(arg.equalsIgnoreCase("shutdown"))
			{
			  String p = parms.getProperty("login", "§§");
			  if(p.equals("§§") || p.equals(""))
			    return exit(prefix+"4: Permissions error: Wrong username!");
			  String pw = parms.getProperty("password", null);
			  if(pw == null)
				return exit(prefix+"5: Permissions error: Wrong password!");
			  if(ram.sessions.containsKey(p))
			  {
			    if(!ram.sessions.get(p).hash.equals(pw))
			      return exit(prefix+"5: Permissions error: Wrong password!");
			  }
			  else
				return exit(prefix+"5: Permissions error: Wrong password!");
			  ram.shutdown = true;
			  sb.append("\"shutdown\", ");
			}
			else
			  return exit(prefix+"3: Invalid input: '"+arg+"'!");
			int l = sb.length();
			if(l > 4)
			  sb.delete(l - 2, l);
			sb.insert(0, "input");
		  }
		}
		else if(codeblock.toLowerCase().startsWith("time(") &&
				codeblock.toLowerCase().endsWith(")"))
		{
		  String world = codeblock.substring(codeblock.indexOf('(') + 1, codeblock.indexOf(')'));
		  if(ram.worldTimes.containsKey(world))
		  {
			sb.append("\"");
			sb.append(ram.worldTimes.get(world));
			sb.append("\"");
			sb.insert(0, "time");
		  }
		  else
			return exit(prefix+"1: '"+world+"' not found!");
		}
		else
		{
		  boolean found = false;
		  for(Entry<String, Function> e: ram.functions.entrySet())
		  {
			String fn = e.getKey();
			String[] args = null;
			if(codeblock.equalsIgnoreCase(fn))
			{
			  found = true;
			  args = new String[0];
			}
			else if(codeblock.toLowerCase().startsWith(fn+"(") &&
					codeblock.toLowerCase().endsWith(")")
			  )
			{
			  found = true;
			  String ret = codeblock.substring(codeblock.indexOf('(') + 1, codeblock.indexOf(')')).replace(" ", "");
			  if(ret.contains(","))
				args = ret.split(",");
			  else
				args = new String[] {ret};
			}
			if(found)
			{
			  replace = e.getValue().onHTTPDExecute(parms, args, ip);
			  if(replace == null)
				return exit(prefix+"2: '"+fn+"' returned null!");
			  found = true;
			  break;
			}
		  }
		  if(!found)
		    return exit(prefix+"0: '"+codeblock+"' not found!");
		}
		ram.lock.set(false);  
		sb.append(");");
		if(replace == null)
		  replace = sb.toString().trim();
		worker = worker.substring(0, start) + replace + worker.substring(end + 3);
		content = content.substring(0, start) + replace + content.substring(end + 3);
	  }
	return content;
  }
  
  private String exit(String ret)
  {
	ram.lock.set(false);
	return ret;
  }
}
